@@ -9,6 +9,9 @@ setClass("ExposureSeries",
          )
 )
 
+#' @export
+setClass("NoExposureSeries", contains="ExposureSeries")
+
 
 
 #' Exposure time-series
@@ -31,17 +34,13 @@ setClass("ExposureSeries",
 #' @return an S4 object of type [ExposureSeries-class]
 #'
 #' @seealso [no_exposure()]
-#' @aliases ExposureSeries-class
+#' @aliases ExposureSeries-class NoExposureSeries-class
 #' @export
 ExposureSeries <- function(series, dates, file, meta, context) {
-  if(!is.data.frame(series))
-    stop("time-series is not a data.frame")
+  check_exposure(series)
   # coerce to data.frame to avoid issues with data.frame-like types
-  series <- as.data.frame(series)
-  if(ncol(series)!=2)
-    stop("time-series must have two columns")
-  if(any(!is.numeric(series[,1])) | any(!is.numeric(series[,2])))
-    stop("time-series contains non-numeric entries")
+  series <- dplyr::select(as.data.frame(series), c(1, 2))
+
   if(missing(file))
     file <- "unknown"
   if(missing(meta))
@@ -72,18 +71,10 @@ ExposureSeries <- function(series, dates, file, meta, context) {
 #' @seealso [set_noexposure()]
 #' @export
 #' @examples
-#' # this will reset the output times of the sample scenario,
-#' # simulate() will quit with an error
-#' try(
-#'   minnow_it %>%
-#'     set_exposure(no_exposure()) %>%
-#'     simulate()
-#' )
-#'
-#' # set zero exposure, but keep original output times
+#' # Set exposure to zero, but keep the original output times
 #' minnow_it %>%
-#'   set_exposure(no_exposure(), reset_times=FALSE) %>%
+#'   set_noexposure() %>%
 #'   simulate()
 no_exposure <- function() {
-  ExposureSeries(data.frame(time=0, conc=0), file="no exposure")
+  new("NoExposureSeries", series=data.frame(time=0, conc=0), file="no exposure")
 }
