@@ -1,3 +1,11 @@
+get_endpoints <- function(x) {
+  if(is.vector(x))
+    return(lapply(x, get_endpoints))
+
+  if(is_sequence(x))
+    return(get_endpoints(x[[1]]))
+  x@endpoints
+}
 
 # Get forcings
 get_forcings <- function(scenario) {
@@ -17,6 +25,18 @@ get_vars <- function(x) {
   return(names(x@init))
 }
 
+# Get settings for moving windows
+get_window <- function(x) {
+  if(is.vector(x) & length(x) > 1) {
+    return(lapply(x, get_window))
+  }
+  if(is_scenario(x)) {
+    return(list(length=x@window.length, interval=x@window.interval))
+  } else if(is_sequence(x)) {
+    return(get_window(x[[1]]))
+  }
+  stop("Type not supported")
+}
 
 #' Get model name
 #'
@@ -49,34 +69,3 @@ get_model_name <- function(x) {
   return(get_model(x))
 }
 
-#' Get scenario tag
-#'
-#' Returns the user-defined, custom tag of a scenario, if available. Tags
-#' can be helpful to quickly distinguish scenarios of the same model type.
-#' The function supports vectorized inputs.
-#'
-#' @param x (vector of) [scenarios] or [`parameter_set`] objects
-#'
-#' @return vector of `character`
-#' @aliases get_tag,ANY-method get_tag,list-method get_tag,EffectScenario-method
-#'   get_tag,ParameterSet-method
-#' @seealso [set_tag()]
-#' @export
-#'
-#' @examples
-#' # returns `fathead minnow`
-#' get_tag(minnow_it)
-#'
-#' # update or set a tag
-#' myscenario <- minnow_it %>% set_tag("My Custom Tag")
-#' # returns `My Custom Tag`
-#' get_tag(myscenario)
-setGeneric("get_tag", function(x) standardGeneric("get_tag"))
-#' @export
-setMethod("get_tag", "ANY", function(x) stop("scenario tag not supported for given object type"))
-#' @export
-setMethod("get_tag", "list", function(x) sapply(x, get_tag))
-#' @export
-setMethod("get_tag", "EffectScenario", function(x) return(x@tag))
-#' @export
-setMethod("get_tag", "ParameterSet", function(x) return(x@tag))
