@@ -19,6 +19,9 @@ has_exposure <- function(x) {
 # forcings we only need to simulate a single moving window instead of all of
 # them. This is due to the fact that results for all windows will be identical.
 has_constant_forcings <- function(x) {
+  # the heuristic would be too complicated/fragile to perform for sequences
+  if(any(is_sequence(x)))
+    return(FALSE)
   if(length(x) > 1)
     return(sapply(x, has_constant_forcings))
 
@@ -45,6 +48,8 @@ has_transfer <- function(x) {
   if(length(x) > 1)
     return(sapply(x, has_transfer))
 
+  if(!is(x, "Transferable"))
+    return(FALSE)
   if(has_regular_transfer(x))
     return(TRUE)
   has_irregular_transfer(x)
@@ -54,6 +59,8 @@ has_regular_transfer <- function(x) {
   if(length(x) > 1)
     return(sapply(x, has_regular_transfer))
 
+  if(!is(x, "Transferable"))
+    return(FALSE)
   x@transfer.interval > 0
 }
 
@@ -61,6 +68,8 @@ has_irregular_transfer <- function(x) {
   if(length(x) > 1)
     return(sapply(x, has_irregular_transfer))
 
+  if(!is(x, "Transferable"))
+    return(FALSE)
   length(x@transfer.times) > 0
 }
 
