@@ -1,3 +1,7 @@
+# TODO rename and overload profile() function
+# TODO remove parameter 'bounds'
+# TODO accept a cvasi_fit object as arg 'x'
+
 #' Likelihood profiling
 #'
 #' @description
@@ -23,15 +27,38 @@
 #' To prevent very small stepsizes when the value goes towards zero (as can
 #' be the case for effect thresholds), an absolute minimum
 #' stepsize (`f_step_abs`), which is specified as a fraction of the best
-#' parameter value (`Xhat`) (unless it is zero, then algoritm takes
+#' parameter value (`Xhat`) (unless it is zero, then the algorithm takes
 #' something small).
 #'
+#' Note that the likelihood of the model given the data can be calculated across
+#' all datasets provided in the calibration set `x`, or calculated separately for
+#' each individual dataset before being combined into one likelihood (by adjusting
+#' the optional parameter `individual`). The latter
+#' has the advantage that different datasets can be given different weights in
+#' the likelihood calculation (using the "weight" slot of the [caliset] objects,`x`).
+#' Further, for continuous data (e.g. biomass), the likelihood considers the variance (standard
+#' deviation) in the log likelihood calculation, which can vary between datasets
+#' when the likelihood is calculated for each dataset separately before combining
+#' into an overall likelihood. The latter could be relevant when factors might lead
+#' to variability between datasets (e.g. different labs, different animal culture,...)
+#'
+#' To conduct the likelihood calculations on separate datasets, the parameter `individual`
+#' which by default is 'FALSE' can be set to 'TRUE'. Then, then log likelihoods
+#' are calculated for each dataset individually (or in subgroups, using the "tag" names of the
+#' [caliset] object, if provided, to group datasets with the same "tag"
+#' before calculating the log likelihood). Subsequently, the log
+#' likelihoods for the subsets are combined into an overall likelihood (considering
+#' the *set* weights provided in the "weight" slot of the [caliset] object).
+#' Note that for each *set* only 1 weight can be provided (i.e. not individual
+#' weights for each datapoint within the *set*), and that *set* with the same tag
+#' should have identical weight.
+#'
 #' The function was inspired by a MatLab BYOM v.6.8 procedure, created by
 #' Tjalling Jager. For details, please refer to BYOM (http://debtox.info/byom.html)
 #' as well as Jager (2021).
 #'
 #' @references
-#' Jager T, 2021. Robust Likelihood-Based Optimization and Uncertainty Analysis
+#' Jager T, 2021: Robust Likelihood-Based Optimization and Uncertainty Analysis
 #' of Toxicokinetic-Toxicodynamic Models. Integrated Environmental Assessment and
 #' Management 17:388-397. \doi{10.1002/ieam.4333}
 #'
@@ -42,39 +69,43 @@
 #' @param data only needed if `x` is a [scenario]
 #' @param bounds optional list of lists (including lower and upper bound): uses defaults in `x` object, but
 #'  can be overwritten here (e.g. bounds <- list(k_resp = list(0,10), k_phot_max = list(0,30)) )
-#' @param refit if 'TRUE' (default), refit if a better minimum is found
-#' @param type "fine" or "coarse" (default) likelihood profiling
-#' @param break_prof if 'TRUE' (default), stop the profiling if a better optimum is located
-#' @param ... additional parameters passed on to [stats::optim()] and [calibrate()]
-#'
+#' @param refit if `TRUE` (default), refit if a better minimum is found
+#' @param type `"fine"` or `"coarse"` (default) likelihood profiling
+#' @param individual if `FALSE` (default), the log likelihood is calculated across
+#' the whole dataset. Alternatively, if `TRUE`, log likelihoods are calculated for
+#' each (group of) *set*(s) individually.
+#' @param break_prof If `TRUE`, then stop the profiling if a better optimum is located.
+#'    Default is `FALSE`.
+#' @param log_scale `FALSE` (default), option to calculate the log likelihood on a
+#' log scale (i.e., observations and predictions are log transformed during calculation)
+#' @param data_type Character argument, `"continuous"` (default) or `"count"`, to specify the data type
+#' for the log likelihood calculations.
+#' @param ... additional parameters passed on to [calibrate()] and [simulate()]. To avoid
+#'   parameter confusion, use argument `method` to select optimization algorithms
+#'   of `calibrate()` and argument `ode_method` to select numerical integration
+#'   schemes of package `deSolve`.
+#' @autoglobal
 #' @examples
 #' # Example with Lemna model - physiological params
 #' library(dplyr)
 #'
-#' # exposure - control run
-#' exp <- Schmitt2013 %>%
-#'   filter(ID == "T0") %>%
-#'   select(time=t, conc)
-#'
 #' # observations - control run
-#' obs <- Schmitt2013 %>%
-#'   filter(ID == "T0") %>%
-#'    select(t, BM=obs)
+#' obs <- schmitt2013 %>%
+#'   filter(trial == "T0")
 #'
 #' # update metsulfuron
 #' myscenario <- metsulfuron %>%
-#'   set_param(c(k_phot_fix = TRUE,Emax = 1)) %>%
-#'   set_init(c(BM = 12)) %>%
-#'   set_exposure(exp)
+#'   set_param(c(k_phot_fix = TRUE, Emax = 1)) %>%
+#'   set_init(c(BM = 0.0012)) %>%
+#'   set_noexposure() %>%
+#'   set_bounds(list(k_phot_max=c(0, 1)))
 #'
 #' fit <- calibrate(
 #'   x = myscenario,
 #'   par = c(k_phot_max = 1),
 #'   data = obs,
-#'   output = "BM",
-#'   lower=0,
-#'   upper=1,
-#'   method="Brent"
+#'   output = "FrondNo",
+#'   method = "Brent"
 #' )
 #'
 #' # Likelihood profiling
@@ -82,7 +113,7 @@
 #' res <- lik_profile(
 #'   x = myscenario,
 #'   data = obs,
-#'   output = "BM",
+#'   output = "FrondNo",
 #'   par = fit$par,
 #'   bounds = list(
 #'     k_phot_max = list(0, 30)
@@ -92,10 +123,10 @@
 #'   method = "Brent"
 #' )
 #' # plot
-#' plot_lik_profile(res)
+#' plot(res)
 #' }
 #'
-#' @return A list containing, for each parameter profiled, the likelihood
+#' @returns A list containing, for each parameter profiled, the likelihood
 #' profiling results as a dataframe;
 #' the 95% confidence interval; the original parameter value; the likelihood plot object; and
 #' the recalibrated parameter values (in case a lower optimum was found)
@@ -117,8 +148,11 @@ lik_profile <- function(x,
                         bounds = NULL,
                         refit = TRUE,
                         type = c("coarse", "fine"),
+                        individual = FALSE,
                         break_prof = FALSE, # typically we do not want to stop
                         # profiling until we have all parameters, and then make a decision based on parameter space
+                        log_scale = FALSE,
+                        data_type = c("continuous", "count"),
                         ...) {
   # check inputs
   # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -126,21 +160,30 @@ lik_profile <- function(x,
   if (length(x) == 1) {
     if (is_scenario(x)) {
       if (is.null(data)) {
-        stop("Scenario provided, but data is missing")
+        stop("Scenario provided, but argument 'data' is missing")
       } else {
-        message("Scenario converted to calibration set")
-        x <- list(caliset(x, data))
+        #message("Scenario converted to calibration set")
+        if(is.data.frame(data))
+          data <- tox_data(data)
+        x <- td2cs(x, data, output_var=output)
       }
     }
   }
-  # check correct input parameters
-  Check_inputs_lik_prof(par = par,
-                        x = x,
-                        output = output,
-                        type = type)
 
-  # check if type is one of the 2 options
+  # check if type and data_type is one of the 2 options
   type <- match.arg(type)
+  data_type <- match.arg(data_type)
+
+  # check correct input parameters
+  check_inputs_lik_prof(
+    par = par,
+    x = x,
+    output = output,
+    type = type,
+    individual = individual,
+    data_type = data_type
+  )
+
 
   # update scenario of calibrationset with the provided par
   for (i in seq_along(x)) {
@@ -159,7 +202,7 @@ lik_profile <- function(x,
   )
 
   # replace defaults if custom boundaries are given
-  if(!is.null(bounds)) {
+  if (!is.null(bounds)) {
     check_bounds(bounds)
     pfree$bounds[names(bounds)] <- bounds
   }
@@ -167,26 +210,20 @@ lik_profile <- function(x,
   # check that each parameter to profile has boundaries set
   missing <- setdiff(names(par), names(pfree$bounds))
   if(length(missing) > 0) {
-    cli::cli_abort("parameter boundaries missing for parameter{?s} {missing}")
+    stop("Parameter boundaries missing for parameter", ifelse(length(missing)>1, "s", ""), ": ",
+         paste(missing, collapse=", "))
   }
 
   # Settings for profiling
   # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-  # cutoffs for the Chi-square
-  # these are the values from a standard chi-square distribution table
-  # (e.g. https://www.itl.nist.gov/div898/handbook/eda/section3/eda3674.htm),
-  # for df 1 to 10, at a alpha = 0.05 significance level,
-  # these values are used in the chi-square test to determine if a fit is significantly different from a previous one (i.e. larger than the cutoff given here)
-  chitable <- c(3.8415, 5.9915, 7.8147, 9.4877, 11.070, 12.592, 14.067, 15.507, 16.919, 18.307)
-  if (length(pnames) > length(chitable)) {
-    stop("more free parameters than function can handle, reduce to 10")
-  } else {
-    chi_crit_j <- chitable[length(pnames)]
-    chi_crit_s <- chitable[1] # when profiling, we have nested models, differing
-    # by 1 parameter (the profiled parameter which is fixed). The likelihood ratio
-    # of such nested models follows a Chi-square distribution with 1 degree of freedom
-  }
+  # cutoffs for the Chi-square, at a alpha = 0.05 significance level,
+  # these values are used in the chi-square test to determine if a fit is
+  # significantly different from a previous one (i.e. larger than the cutoff given here)
+  chi_crit_j <- stats::qchisq(p=0.95, df=length(pnames))
+  chi_crit_s <- stats::qchisq(p=0.95, df=1) # when profiling, we have nested models, differing
+  # by 1 parameter (the profiled parameter which is fixed). The likelihood ratio
+  # of such nested models follows a Chi-square distribution with 1 degree of freedom
 
 
   # setting for profile type
@@ -219,6 +256,16 @@ lik_profile <- function(x,
 
   # a list to store results
   res_list <- list()
+  # (important) arguments to this function
+  arg_list = list(
+    "refit"=refit,
+    "type"=type,
+    "individual"=individual,
+    "log_scale"=log_scale,
+    "data_type"=data_type,
+    "chi_crit_s"=chi_crit_s
+  )
+  attr(res_list, "args") <- arg_list
 
   # profile parameter by parameter
   for (par_select in pnames) {
@@ -238,6 +285,9 @@ lik_profile <- function(x,
       l_crit_min = l_crit_min,
       l_crit_stop = l_crit_stop,
       refit = refit,
+      individual = individual,
+      log_scale = log_scale,
+      data_type = data_type,
       ...
     ))
 
@@ -246,8 +296,7 @@ lik_profile <- function(x,
       if (any(res_list[[par_select]]$likelihood_profile[, "log_lik_rat"] < 0.00)) {
         # give warning to user
         warning(paste0("Better parameter value found for ", par, ", profiling halted"))
-        class(res_list) <- c(class(res_list), "lik_profile")
-        return(res_list)
+        break
       }
     }
   } # end of parameter for loop
@@ -269,13 +318,21 @@ lik_profile <- function(x,
 # @param par named vector - parameters (names and values) to be profiled
 # @param output character vector - the output from the [scenario] or [caliset] that is used in calibration
 # @param type "fine" or "coarse" (default) likelihood profiling
+# @param individual if 'FALSE' (default), the log likelihood is calculated across
+# the whole dataset. Alternatively, if 'TRUE', log likelihoods are calculated for
+# each (group of) *set*(s) individually.
+# @param data_type Character argument, "continuous" (default) or "count", to specify the data type
+# for the log likelihood calculations.
 #
 # @return x as a list of [caliset], and objects error message when needed
 
-Check_inputs_lik_prof<- function(par,
-                      x,
-                      output,
-                      type){
+check_inputs_lik_prof <- function(par,
+                                  x,
+                                  output,
+                                  type,
+                                  individual,
+                                  data_type) {
+
   # check if attempt to profile more params than possible ~~~~~~~~~~~~~~~~~~~
   if (length(par) > 10) {
     stop("attempt to profile more parameters that function allows, reduce no. of parameters")
@@ -285,32 +342,77 @@ Check_inputs_lik_prof<- function(par,
   # check if Calibrationset or EffectScenario is provided, and convert EffectScenario to CalibrationSet
   if (is.list(x)) {
     if (is(x[[1]]) != "CalibrationSet") {
-      warning("incorrect model specified, please provide a scenario or a list of calibration sets")
-      stopifnot(any(is(x[[1]]) == "CalibrationSet"))
+      stop("incorrect model specified, please provide a scenario or a list of calibration sets")
     }
   } else { ## input is not a list
     if (is_scenario(x)) {
       # all fine
     } else { # input is not a list, and also not an EffectScenario
-      warning("incorrect model specified, please provide a scenario or a list of calibration sets")
-      stopifnot(is_scenario(x))
+      stop("incorrect model specified, please provide a scenario or a list of calibration sets")
     }
   }
 
+  # check if the obs match the data_type
+  all <- lapply(x, slot, name = "data")
+  observed_data_type <- unlist(lapply(all, function(x) is(x[[2]])))
+  stopifnot(any(observed_data_type =="numeric" ))
+  if(data_type == "count"){
+    stopifnot(any(observed_data_type == "integer"))
+  }
+
   # check par ~~~~~~~~~~~~~~~~~~~
   # check if parameters are provided as named vector
   stopifnot(mode(par) == "numeric")
   if (is.null(names(par))) {
-    warning("parameters are not provided as a named vector")
-    stopifnot(!is.null(names(par)))
+    stop("parameters are not provided as a named vector")
   }
 
   # check output ~~~~~~~~~~~~~~~~~~~
   stopifnot(mode(output) == "character")
 
+
+  # check data weights and tags if "individual = TRUE" is chosen ~~~~~~~~~~~~~~~
+  if (individual == TRUE) {
+    data_weights <- lapply(x, slot, name = "weight")
+    data_tag <- unlist(lapply(x, slot, name = "tag"))
+
+    # checks to ensure data weights are given properly
+    if (length(data_weights) != length(x)) {
+      stop("Please ensure all datasets have a weight assigned to them")
+    }
+
+    if (any(unlist(lapply(data_weights, length)) > 1)) {
+      stop("Please ensure each individual datasets has only 1 weight assigned to them")
+    }
+
+    if (any(is.na(unlist(data_weights)))) {
+      stop("Please ensure all datasets have a weight assigned to them (NA is not allowed)")
+    }
+
+    # checks to ensure appropriate tags are given
+    if (!is.null(data_tag)) {
+      for (i in seq_along(unique(data_tag))) {
+        index <- which(data_tag == unique(data_tag)[i])
+        if (length(unique(unlist(data_weights[index]))) != 1) {
+          stop("Please ensure datasets with the same tag, have the same weight assigned to them")
+        }
+      }
+    } else { # if no data_tags are given, make sure that all weights are equal
+      if (length(unique(unlist(data_weights))) != 1) {
+        stop("Please ensure tag names are provided, when weights are assigned to individual data subsets")
+      }
+    }
+  }
+  else {
+    data_weights <- as.vector(sapply(x, slot, name="weight"))
+    if(length(unique(data_weights)) != 1) {
+      warning("Assigned dataset weights will be ignored for profiling if argument `individual=FALSE` ...")
+    }
+  }
 }
 
 
+# TODO make print statements optional
 #  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 # Parameter profiling
 #
@@ -320,11 +422,12 @@ Check_inputs_lik_prof<- function(par,
 # data calculated (using function [log_lik()]).
 #
 # The function was inspired by the MatLab BYOM procedure by
-# Tjalling Jager, http://debtox.info/byom.html, as described in Jager 2021: Robust Likelihood-Based Optimization and Uncertainty Analysis of Toxicokinetic-
-# Toxicodynamic Models" Integrated Environmental Assessment and Management 17:388-397
-# DOI: 10.1002/ieam.4333
+# Tjalling Jager, http://debtox.info/byom.html, as described in Jager 2021:
+# Robust Likelihood-Based Optimization and Uncertainty Analysis of Toxicokinetic-
+# Toxicodynamic Models" Integrated Environmental Assessment and Management
+# 17:388-397 DOI: 10.1002/ieam.4333
 #
-# @param par `character`, the parameter to be profiled
+# @param par_select `character`, the parameter to be profiled
 # @param x list of [caliset] objects
 # @param pfree list of parameter values and their bounds for the profiled parameter
 # @param type "fine" or "coarse" (default) likelihood profiling
@@ -337,17 +440,24 @@ Check_inputs_lik_prof<- function(par,
 # @param l_crit_min, `numeric`,minimum change in likelihood, below this value, stepsize is increased
 # @param l_crit_stop, `numeric`,stop when likelihood ratio reaches this value
 # @param refit if 'TRUE' (default), refit if a better minimum is found
+# @param individual if 'FALSE' (default), the log likelihood is calculated across
+# the whole dataset. Alternatively, if 'TRUE', log likelihoods are calculated for
+# each (group of) *set*(s) individually.
+# @param log_scale 'FALSE' (default), option to calculate the log likelihood on a
+# log scale (i.e., observations and predictions are log transformed during calculation)
+# @param data_type Character argument, "continuous" (default) or "count", to specify the data type
+# for the log likelihood calculations.
 # @param ... additional parameters passed on to [stats::optim()] and [calibrate()]
 #
 # @return A list of containing the likelihood profiling results as a dataframe;
 # the 95% confidence interval; the original parameter value; the likelihood plot object; and
 # the recalibrated parameter values (in case a lower optimum was found)
-#' @global start
+#' @autoglobal
 profile_par <- function(par_select,
                         x, pfree, type, output,
                         max_iter, f_step_min, f_step_max,
                         chi_crit_j, chi_crit_s, l_crit_max, l_crit_min, l_crit_stop,
-                        refit, ...) {
+                        refit, individual, log_scale, data_type, ...) {
   # initialize
   # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
@@ -361,20 +471,49 @@ profile_par <- function(par_select,
   # get best fit value (i.e., calibrated value)
   x_hat_orig <- pfree$values[[par_select]]
 
-  # calulate log likelihood with original model
-  pred_orig <- list()
-  ll_orig <- list()
-  for (i in seq_along(x)) {
-    pred_orig[[i]] <- x[[i]]@scenario %>%
-      set_times(x[[i]]@data[, 1]) %>% # time is the 1st column, mandatory that it is the 1st column
-      simulate()
-    ll_orig[[i]] <- log_lik(
+  # get all data, and weights
+  all_data <- lapply(x, slot, name = "data")
+  data_tag <- unlist(lapply(x, slot, name = "tag"))
+  names(all_data) <- data_tag
+  data_weights <- unlist(lapply(x, slot, name = "weight"))
+
+  # Make predictions with the original model
+  rs <- eval_cs(x, output=output, verbose=FALSE, .ignore_method=TRUE, ...)
+
+  # Calulate log likelihood with original model
+  #   Option 1: calculation of loglik across all datasets
+  if (individual == FALSE) {
+    # calc log lik
+    ll_orig <- log_lik(
       npars = length(pfree$values), # Note: only the free params (i.e. ones that you did the calibration on)
-      obs = x[[i]]@data[, 2], # observations are the 2nd column, mandatory that it is the 2nd column
-      pred = pred_orig[[i]][, c(output)]
+      obs = rs$obs,
+      pred = rs$pred,
+      log_scale = log_scale,
+      data_type = data_type
     )
+  } else {
+    #  Option 2: calculation for individual sub-datasets, which are then combined
+    ll_list <- list()
+    subsets <- data.frame(obs=rs$obs,
+                     pred=rs$pred,
+                     wgts=rs$wgts,
+                     tags=unlist(rs$tags)) %>%
+      dplyr::group_by(tags) %>%
+      dplyr::group_split()
+
+    for (i in seq_along(subsets)) {
+      ll_list[[i]] <- log_lik(
+        npars = length(pfree$values), # Note: only the free params (i.e. ones that you did the calibration on)
+        obs = subsets[[i]]$obs,
+        pred = subsets[[i]]$pred,
+        log_scale = log_scale,
+        data_type = data_type
+      ) * unique(subsets[[i]]$wgts)
+    }
+    ll_orig <- sum(unlist(ll_list))
   }
 
+
   # additional setting for profile type
   # values taken from BYOM
   if (type == "fine") {
@@ -391,7 +530,7 @@ profile_par <- function(par_select,
     }
   }
 
-  message("start param value: ", round(x_hat_orig, 3), "LL:", round(sum(unlist(ll_orig)), 3))
+  message("start param value: ", round(x_hat_orig, 3), "\nLL:", round(ll_orig, 3))
 
   # refit the model with the par fixed to lower values, and then higher values
   # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -400,7 +539,7 @@ profile_par <- function(par_select,
   log_lik_rat <- c(0)
   par_value <- c(x_hat_orig)
   results <- list(
-    loglik = sum(unlist(ll_orig)),
+    loglik = ll_orig,
     log_lik_rat = log_lik_rat,
     par_value = par_value
   )
@@ -465,39 +604,51 @@ profile_par <- function(par_select,
     prof_param_index <- which(names(pfree[["values"]]) == par_select)
     pfree_remaining <- unlist(pfree[["values"]])[-prof_param_index]
 
-    # # recalibrate
-    # fit_new <- calibrate(
-    #   x = x,
-    #   par = pfree_remaining,
-    #   output = output
-    # )
+    # Make predictions with the newly fitted model
+    rs <- eval_cs(x, output=output, verbose=FALSE, .ignore_method=TRUE, ...)
 
-
-    # predict and calulate log likelihood with newly fitted model
-    pred_new <- list()
-    ll_new <- list()
-
-    for (i in seq_along(x)) {
-      # pred_new[[i]] <- simulate(model[[i]]@scenario)
-      pred_new[[i]] <- x[[i]]@scenario %>%
-        set_times(x[[i]]@data[, 1]) %>% # needs to have the same timepoint as the data
-        simulate()
-      ll_new[[i]] <- log_lik(
-        npars = length(pfree_remaining),
-        obs = x[[i]]@data[, 2],
-        pred = pred_new[[i]][, output]
+    # Calulate log likelihood with the newly fitted model
+    #   Option 1: calculation of loglik across all datasets
+    if (individual == FALSE) {
+      # calc log lik
+      ll_new <- log_lik( # list with only 1 entry
+        npars = length(pfree$values), # Note: only the free params (i.e. ones that you did the calibration on)
+        obs = rs$obs,
+        pred = rs$pred,
+        log_scale = log_scale,
+        data_type = data_type
       )
+    } else {
+      #  Option 2: calculation for individual sub-datasets, which are then combined
+      ll_list <- list()
+      subsets <- data.frame(obs=rs$obs,
+                            pred=rs$pred,
+                            wgts=rs$wgts,
+                            tags=unlist(rs$tags)) %>%
+        dplyr::group_by(tags) %>%
+        dplyr::group_split()
+
+      for (i in seq_along(subsets)) {
+        ll_list[[i]] <- log_lik(
+          npars = length(pfree$values), # Note: only the free params (i.e. ones that you did the calibration on)
+          obs = subsets[[i]]$obs,
+          pred = subsets[[i]]$pred,
+          log_scale = log_scale,
+          data_type = data_type
+        ) * unique(subsets[[i]]$wgts)
+      }
+      ll_new <- sum(unlist(ll_list))
     }
 
     # save new LL
-    results[["loglik"]][iter] <- sum(unlist(ll_new))
+    results[["loglik"]][iter] <- ll_new
 
     # save -2*ln(likelihood ratio) comparing the full (original) model with the nested (1 param fixed) model
-    results[["log_lik_rat"]][iter] <- -2 * (results[["loglik"]][iter] - sum(unlist(ll_orig))) # matlab BYOM L. 704
+    results[["log_lik_rat"]][iter] <- -2 * (ll_new - ll_orig) # matlab BYOM L. 704
 
     # difference with ll from previous iteration (needed to decide on step size etc.) # matlab BYOM L. 705
     if (iter == 1) {
-      delta_l <- abs(results[["loglik"]][iter] - sum(unlist(ll_orig)))
+      delta_l <- abs(results[["loglik"]][iter] - ll_orig)
     } else {
       delta_l <- abs(results[["loglik"]][iter] - results[["loglik"]][iter - 1])
     }
@@ -569,7 +720,7 @@ profile_par <- function(par_select,
     min_ind <- which(low_df$log_lik_rat == min(low_df$log_lik_rat))
     low_df <- low_df[1:min_ind, ]
   }
-  if (nrow(low_df) > 1 & !all(low_df[, "log_lik_rat"] == 0)) {
+  if (nrow(low_df) > 1 && !all(low_df[, "log_lik_rat"] == 0)) {
     # needs to be at least 2 values, at least one with a ll not 0
     f_inter <- stats::approxfun(
       y = low_df$par_value,
@@ -588,7 +739,7 @@ profile_par <- function(par_select,
     min_ind <- which(up_df$log_lik_rat == min(up_df$log_lik_rat))
     up_df <- up_df[min_ind:nrow(up_df), ]
   }
-  if (nrow(up_df) > 1 & !all(up_df[, "log_lik_rat"] == 0)) {
+  if (nrow(up_df) > 1 && !all(up_df[, "log_lik_rat"] == 0)) {
     # needs to be at least 2 values, at least one with a ll not 0
     f_inter <- stats::approxfun(
       y = up_df$par_value,
@@ -602,56 +753,12 @@ profile_par <- function(par_select,
     } # in case multiple values with ll of 0, take the max param value
   }
 
-
-
-  # plot
-  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  prof_plot <- res_df %>%
-    ggplot2::ggplot() +
-    # # zone of acceptabiity
-    # ggplot2::geom_rect(
-    #   ggplot2::aes(
-    #     xmin = -Inf, xmax = Inf,
-    #     ymin = min(log_lik_rat), ymax = chi_crit_s
-    #   ),
-    #   fill = "darkolivegreen3", alpha = 0.2
-    # ) +
-    ggplot2::annotate("rect",
-      xmin = -Inf, xmax = Inf,
-      ymin = min(res_df$log_lik_rat), ymax = chi_crit_s,
-      alpha = 0.3, fill = "darkolivegreen3"
-    ) +
-    ggplot2::geom_hline(yintercept = chi_crit_s, color = "tomato", lwd = 1.2) +
-    # points for values
-    ggplot2::geom_point(ggplot2::aes(
-      x = par_value,
-      y = log_lik_rat,
-      color = start
-    ), size = 2) +
-    ggplot2::scale_color_manual(values = c("black", "orange")) +
-    # connect points
-    ggplot2::geom_line(ggplot2::aes(
-      x = par_value,
-      y = log_lik_rat
-    )) +
-    # could add 95% CI
-    ggplot2::geom_vline(
-      xintercept = c(low95, up95),
-      linetype = "dashed",
-      color = "grey30", lwd = .5
-    ) +
-    ggplot2::xlab(par_select) +
-    # consmetics
-    ggplot2::scale_y_continuous(limits = c(min(res_df$log_lik_rat), NA), expand = c(0, 0)) +
-    ggplot2::theme_classic()
-
   # save all outputs for the parameter
   param_res <- list(
     likelihood_profile = res_df,
     confidence_interval = c(low95, up95),
     prof_region = c(min, max),
     orig_par_value = x_hat_orig,
-    plot = prof_plot,
     fit_new = NULL
   )
 
@@ -660,7 +767,7 @@ profile_par <- function(par_select,
 
   # Check in results if any log likelihood ratio is lower than the original
   if (refit == TRUE) {
-    if (any(res_df[["loglik"]] > sum(unlist(ll_orig)) + 0.01)) {
+    if (any(res_df[["loglik"]] > ll_orig + 0.01)) {
       # only if the difference is at least 0.01 as in BYOM (value taken from BYOM)
       # this ignores tiny, irrelevant, improvement
       best_par_value <- res_df[which(res_df$log_lik_rat == min(res_df$log_lik_rat)), "par_value"]
@@ -674,7 +781,9 @@ profile_par <- function(par_select,
         x = x,
         par = pfree_remaining,
         output = output,
-        verbose = FALSE
+        log_scale = log_scale,
+        verbose = FALSE,
+        ...
       )
       # save recalibrated result
       param_res[["fit_new"]] <- list(
@@ -689,38 +798,40 @@ profile_par <- function(par_select,
 } # end of profile_par function
 
 
-#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-#' Plot likelihood profiles or all profiled parameters
-#'
-#' @description The function provides a combined plot of the likelihood profiles
-#' of all parameters profiled.
-#'
-#' @param x object of class lik_profile
-#'
-#' @return plots
-#' @export
-plot_lik_profile <- function(x) {
-  stopifnot("lik_profile" %in% class(x))
-  npars <- length(x)
-  n1 <- ceiling(sqrt(npars))
-  do.call(gridExtra::grid.arrange, c(lapply(x, function(x) x$plot), list(nrow = n1)))
-}
-
 #' Calculate log likelihood
 #'
 #' @description Calculates the sum of log likelihoods of each observation given
-#' the model parameterization (considering a normal distribution around the prediction
-#' for each datapoint)
+#' the model parameterization.
+#'
+#' Current implementations enable log likelihood calculations for:
+#' 1) continuous data, considering a normal distribution around the prediction
+#' for each datapoint,
+#' 2) count data, considering a multinomial distribution for data reporting the
+#' number of survivors over time.
+#'
+#' The log likelihood calculation for count data was inspired by a MatLab BYOM v.6.8
+#' procedure, created by Tjalling Jager. For details, please refer to BYOM
+#' (http://debtox.info/byom.html) as well as Jager (2021).
+#'
+#' @references
+#' Jager T, 2021. Robust Likelihood-Based Optimization and Uncertainty Analysis
+#' of Toxicokinetic-Toxicodynamic Models. Integrated Environmental Assessment and
+#' Management 17:388-397. \doi{10.1002/ieam.4333}
 #'
-#' @param npars named numeric vector of parameters that the model was calibrated on
 #' @param obs numeric vector of observed values
 #' @param pred numeric vector of predicted values
+#' @param data_type determines the if likelihood profiling is conducted for
+#' `"continuous"` (default) or `"count"` data
+#' @param log_scale `FALSE` (default), option to calculate the log likelihood on a
+#' log scale (i.e., observations and predictions are log transformed during calculation)
+#' @param npars named numeric vector of parameters that the model was calibrated on,
+#' required for `"continuous"` data type, optional for `"count"`.
 #'
 #' @return the log likelihood value
 #' @export
 #'
 #' @examples
-#'
+#' # simple example for continuous data #####
 #' # observations
 #' obs <- c(12, 38, 92, 176, 176, 627, 1283, 2640)
 #' # intercept, a, and slope, b, of a Poisson regression fitted through obs
@@ -731,24 +842,91 @@ plot_lik_profile <- function(x) {
 #' plot(seq(1:length(obs)), obs)
 #' lines(seq(1:length(obs)), pred)
 #' log_lik(
-#'   npars = length(pars),
 #'   obs = obs,
-#'   pred = pred
+#'   pred = pred,
+#'   npars = length(pars),
 #' )
 #'
-log_lik <- function(npars, obs, pred){
+#' # example with count data and GUTS model #####
+#' library(dplyr)
+#' # observational data
+#' dt <- ringtest_c %>% filter(replicate == "E")
+#' myexposure <- dt %>% select(time, conc)
+#' obs <- dt %>%
+#'   mutate(S=Nsurv / max(Nsurv)) %>%
+#'   select(time, S)
+#' # GUTS model
+#' GUTS_RED_IT() %>%
+#'   set_param(c(hb = 0)) %>%
+#'   set_exposure(myexposure) -> myscenario
+#' # fit
+#' fit <- calibrate(
+#'   x = myscenario,
+#'   par = c(kd=1.2, alpha=9.2, beta=4.3),
+#'   data = obs,
+#'   output = "S")
+#' # update
+#' myscenario <- myscenario %>% set_param(fit$par)
+#' # simulate
+#' pred <- myscenario %>% simulate()
+#' pred <- pred$S #* max(obs$S)
+#' obs <- obs$S
+#' # calc likelihood
+#' log_lik(obs,
+#'     pred,
+#'     data_type = "count")
+#'
+
+log_lik <- function(obs,
+                    pred,
+                    data_type = c("continuous", "count"),
+                    log_scale = FALSE,
+                    npars = NULL){
+
+  # general checks
   stopifnot(length(obs) == length(pred))
-  stopifnot(length(npars) == 1)
-  stopifnot(is.numeric(npars))
+  if(any(is.na(c(obs, pred)))){
+    warning("missing values in observations or predictions")
+  }
+  data_type <- match.arg(data_type)
 
+  # transformations
+  if(log_scale == TRUE){
+    obs = log(obs)
+    pred = log(pred)
+  }
+
+  # Likelihoods
+  if(data_type == "continuous"){
+
+    # specific checks
+    stopifnot(is.numeric(npars))
+    stopifnot(length(npars) == 1)
+
+    # calculations
+    k <- npars
+    res <- obs - pred
+    n <- length(res)
+    SSE <- sum(res^2)
+    sigma <- sqrt(SSE / (n - k))
+    sigma_unbiased <- sigma * sqrt((n - k) / n)
+
+    # log likelihood for normal probability density
+    LL <- sum(log(stats::dnorm(x = obs, mean = pred, sd = sigma_unbiased)))
+    return(LL)
+  }
+  if(data_type == "count"){
+
+    # calculations
+    Ndeaths <- -diff(obs)       # numbers of deaths
+    Mdeaths <- -diff(pred)      # conditional probabilities of deaths
+    Mdeaths <- max(Mdeaths,1e-50)   # otherwise we get problems taking the logarithm
+    pred    <- max(pred,1e-50)      # otherwise we get problems taking the logarithm
+
+    # log likelihood for count data (multinomial)
+    LL  <- sum( Ndeaths * log(Mdeaths) )
+    return(LL)
+  }
 
-  k <- npars
-  res <- obs - pred
-  n <- length(res)
-  SSE <- sum(res^2)
-  sigma <- sqrt(SSE / (n - k))
-  sigma_unbiased <- sigma * sqrt((n - k) / n)
-  # log likelihood for normal probability density
-  LL <- sum(log(stats::dnorm(x = obs, mean = pred, sd = sigma_unbiased)))
-  return(LL)
 }
+
