@@ -114,7 +114,7 @@ setClass("DebAbj", contains="Deb")
 #'   set_param(c(p_M=3000,v=0.02,k_J=0.6,p_Am=300,kap=0.9,E_G=4000,f=1,
 #'               E_Hj=0.05,E_Hp=0.3,kap_R=0.9,ke=1,c0=0,cT=1,L_b=0.02,
 #'               L_j=0.04,MoA=0)) %>%
-#'   set_exposure(no_exposure()) %>%
+#'   set_noexposure() %>%
 #'   set_times(0:10) %>%
 #'   simulate()
 #'
@@ -132,7 +132,7 @@ DEB_abj <- function() {
       control.req=TRUE,
       init=c("L"=0,"E"=0,"H"=0,"R"=0,"cV"=0,"Lmax"=0),
       exposure=no_exposure()
-  )
+  ) %>% set_noexposure()
 }
 
 
@@ -164,12 +164,12 @@ solver_deb_abj <- function(scenario, method="lsoda", ...) {
   forcings <- list(scenario@exposure@series)
 
   # run solver
-  as.data.frame(ode(y=scenario@init, times=scenario@times, parms=params, method=method,
-                    dllname="cvasi", initfunc="deb_abj_init", func="deb_abj_func",
-                    initforc="deb_abj_forc", forcings=forcings, outnames=outnames, ...))
+  ode(y=scenario@init, times=scenario@times, parms=params, method=method,
+        dllname="cvasi", initfunc="deb_abj_init", func="deb_abj_func",
+        initforc="deb_abj_forc", forcings=forcings, outnames=outnames, ...)
 }
 #' @include solver.R
-#' @describeIn solver Numerically integrates DEB_abj models
+#' @describeIn solver Numerically integrates DEB_abj scenarios
 setMethod("solver", "DebAbj", function(scenario, ...) solver_deb_abj(scenario, ...))
 
 
