@@ -107,13 +107,21 @@ setClass("GutsRedIt", contains="EffectScenario")
 #'
 #' @inherit GUTS-RED-models references
 #'
+#' @section Model history and changes:
+#' - cvasi v1.0.0
+#'   - Model and parameters as described by EFSA PPR (2018).
+#' - cvasi v1.5.0
+#'   - Cumulative damage over time will be calculated on ODE level, instead on
+#'     results of [simulate()], to avoid influence of selected output times
+#'     on predicted survival.
+#'
 #' @param param optional named `list` or `vector` with model parameters
 #' @param init optional named numeric `vector` to use as initial state
 #' @return an S4 object of type [GutsRedIt-class]
 #'
 #' @export
 #' @family GUTS-RED models
-#' @aliases GutsRedIt-class
+#' @aliases GutsRedIt-class GUTS-RED-IT
 #' @importFrom methods new
 GUTS_RED_IT <- function(param, init) {
   new("GutsRedIt",
@@ -122,7 +130,7 @@ GUTS_RED_IT <- function(param, init) {
       endpoints = c("L"),
       init = c(D=0, H=0),
       control.req=FALSE
-  ) -> o
+  ) %>% set_noexposure() -> o
   if(!missing(param))
     o <- set_param(o, param)
   if(!missing(init))
@@ -154,7 +162,7 @@ GUTS_RED_IT <- function(param, init) {
 #'
 #' @export
 #' @family GUTS-RED models
-#' @aliases GutsRedSd-class
+#' @aliases GutsRedSd-class GUTS-RED-SD
 #' @importFrom methods new
 GUTS_RED_SD <- function(param, init) {
   new("GutsRedSd",
@@ -163,7 +171,7 @@ GUTS_RED_SD <- function(param, init) {
       endpoints = c("L"),
       init = c(D=0, H=0),
       control.req=FALSE
-  ) -> o
+  ) %>% set_noexposure() -> o
   if(!missing(param))
     o <- set_param(o, param)
   if(!missing(init))
@@ -187,20 +195,26 @@ solver_gutsredsd <- function(scenario, method="lsoda", hmax=1, ...) {
   if(is.list(params))
     params <- unlist(params)
 
+  # check if all required parameters are present
+  params_missing <- is.na(params[scenario@param.req])
+  if(any(params_missing)) {
+    stop("Missing parameters: ", paste(scenario@param.req[params_missing], collapse=", "))
+  }
+
   # make sure that parameters are present and in required order
   params <- params[c("kd", "hb", "z", "kk")]
 
-  df <- as.data.frame(ode(y=scenario@init, times=scenario@times, parms=params, dllname="cvasi",
-                          initfunc="gutsredsd_init", func="gutsredsd_func", initforc="gutsredsd_forc",
-                          forcings=scenario@exposure@series, outnames=c("Cw"),
-                          method=method, hmax=hmax, ...))
+  df <- ode2df(ode(y=scenario@init, times=scenario@times, parms=params, dllname="cvasi",
+                    initfunc="gutsredsd_init", func="gutsredsd_func", initforc="gutsredsd_forc",
+                    forcings=scenario@exposure@series, outnames=c("Cw"),
+                    method=method, hmax=hmax, ...))
   # Derive survival probability, see EFSA Scientific Opinion on TKTD models, p. 33
   # doi:10.2903/j.efsa.2018.5377
   df$S <- exp(-df$H) # background hazard rate included in H (if enabled)
   df
 }
 #' @include solver.R
-#' @describeIn solver Numerically integrates GUTS-RED-SD models
+#' @describeIn solver Numerically integrates GUTS-RED-SD scenarios
 setMethod("solver", "GutsRedSd", function(scenario, ...) solver_gutsredsd(scenario, ...))
 
 # @param scenario Scenario object
@@ -213,20 +227,34 @@ solver_gutsredit <- function(scenario, method="lsoda", hmax=1, ...) {
   if(is.list(params))
     params <- unlist(params)
 
+  # check if all required parameters are present
+  params_missing <- is.na(params[scenario@param.req])
+  if(any(params_missing)) {
+    stop("Missing parameters: ", paste(scenario@param.req[params_missing], collapse=", "))
+  }
+
   # make sure that parameters are present and in required order
   odeparams <- params[c("kd","hb")]
 
-  df <- as.data.frame(ode(y=scenario@init, times=scenario@times, parms=odeparams, dllname="cvasi",
-                          initfunc="gutsredit_init", func="gutsredit_func", initforc="gutsredit_forc",
-                          forcings=scenario@exposure@series, outnames=c("Cw"),
-                          method=method, hmax=hmax, ...))
+  # CMax := cumulative maximum of D(t)
+  CMax_init <- scenario@init[["D"]]
+  df <- ode2df(ode(y=c(scenario@init, "CMax"=CMax_init), times=scenario@times, parms=odeparams, dllname="cvasi",
+                initfunc="gutsredit_init", func="gutsredit_func", initforc="gutsredit_forc",
+                forcings=scenario@exposure@series, outnames=c("Cw"),
+                method=method, hmax=hmax, ...))
+
+  # The internal damage can, by definition, not become negative. However, the
+  # numerical schemes may sometimes return negative values due to interpolation.
+  df$CMax <- pmax(0, df$CMax)
+
   # Derive survival probability, EFSA Scientific Opinion on TKTD models, p. 33
   # doi:10.2903/j.efsa.2018.5377
-  FS <- (1 / (1 + (cummax(df$D) / params["alpha"])^(-params["beta"])))
+  FS <- (1 / (1 + (df$CMax / params["alpha"])^(-params["beta"])))
   df$S <- (1 - FS) * exp(-df$H)
+  df$CMax <- NULL
   df
 }
-#' @describeIn solver Numerically integrates GUTS-RED-IT models
+#' @describeIn solver Numerically integrates GUTS-RED-IT scenarios
 setMethod("solver", "GutsRedIt", function(scenario, ...) solver_gutsredit(scenario, ...) )
 
 
@@ -242,6 +270,10 @@ fx_gutsredsd <- function(scenario, ...) {
     scenario@param$hb <- 0
 
   res <- simulate(scenario, ...)
+  if(!num_success(res)) {
+    return(c("L"=NA_real_))
+  }
+
   c("L"=1 - tail(res$S, n=1))
 }
 
@@ -249,15 +281,20 @@ fx_gutsredsd <- function(scenario, ...) {
 fx_gutsredit <- function(scenario, ...) {
   # we avoid the control run if we just set the background mortality to zero
   # as it would cancel out anyways
-  if(scenario@param$hb > 0)
-    scenario@param$hb <- 0
+  if(scenario@param$hb > 0) {
+    scenario <- set_param(scenario, c(hb=0))
+  }
 
   res <- simulate(scenario, ...)
+  if(!num_success(res)) {
+    return(c("L"=NA_real_))
+  }
+
   c("L"=1 - tail(res$S, n=1))
 }
 
 #' @include fx.R
-#' @describeIn fx Survival and lethality in [GUTS-RED-models]
+#' @describeIn fx Calculates lethality of [GUTS-RED-SD] scenarios
 setMethod("fx", "GutsRedSd", function(scenario, ...) fx_gutsredsd(scenario, ...))
-#' @describeIn fx Survival and lethality in [GUTS-RED-models]
+#' @describeIn fx Calculates lethality of [GUTS-RED-IT] scenarios
 setMethod("fx", "GutsRedIt", function(scenario, ...) fx_gutsredit(scenario, ...))
