@@ -4,6 +4,7 @@
 
 # Lemna model class (Klein et al. 2021)
 #' @export
+#' @include man-lemna.R
 setClass("LemnaSetac", contains="Lemna")
 # for backwards compatibility
 #' @export
@@ -14,10 +15,10 @@ setClass("LemnaSetacScenario", contains="LemnaSetac")
 ## Constructor
 ########################
 
-#' Lemna model (Klein et al. 2021)
+#' Lemna model v1.3 (Klein et al. 2021)
 #'
 #' The model was described and published by the SETAC Europe Interest Group
-#' Effect Modeling (Klein et al. 2022). It is based on the *Lemna* model
+#' Effect Modeling (Klein et al. 2021). It is based on the *Lemna* model
 #' by Schmitt (2013). The model is a mechanistic combined
 #' toxicokinetic-toxicodynamic (TK/TD) and growth model for the aquatic
 #' macrophytes *Lemna spp.*. The model simulates the development of Lemna biomass
@@ -29,16 +30,21 @@ setClass("LemnaSetacScenario", contains="LemnaSetac")
 #'
 #' @section State variables:
 #' The model has two state variables:
-#' - `BM`, Biomass (g dw m-2)
-#' - `M_int`, Mass of toxicant in plant population (mass per m2, e.g. ug m-2)
+#' - `BM`, Biomass (g dw)
+#' - `M_int`, Mass of toxicant in plant population (ng)
+#'
+#' The  units of the state variables `BM` and `M_int` are to be interpreted
+#' as *per vessel* in laboratory tests, and as *per square meter* for field populations.
+#' The same interpretation applies to all parameters relating biomass and toxicant mass to
+#' other entities.
 #'
 #' @section Model parameters:
 #' - Growth model
 #'   - `k_photo_fixed`, Model switch for unlimited growth conditions (TRUE/FALSE)
 #'   - `k_photo_max`, Maximum photosynthesis rate (d-1)
 #'   - `k_loss`, Reference loss rate (d-1)
-#'   - `BM_threshold`, Lower biomass abundance threshold,  (g dw m-2)
-#'   - `BM_min`, Reservoir for biomass recovery,  (g dw m-2)
+#'   - `BM_threshold`, Lower biomass abundance threshold,  (g dw)
+#'   - `BM_min`, Reservoir for biomass recovery,  (g dw)
 #'
 #' - Temperature response of photosynthesis
 #'   - `T_opt`, Optimum growth temperature (°C)
@@ -58,7 +64,7 @@ setClass("LemnaSetacScenario", contains="LemnaSetac")
 #'   - `P_50`, Half-saturation constant of Phosphorus (mg P L-1)
 #'
 #' - Density dependence of photosynthesis
-#'   - `BM_L`, Carrying capacity (g dw m-2)
+#'   - `BM_L`, Carrying capacity (g dw)
 #'
 #' - Concentration response (Toxicodynamics)
 #'   - `EC50_int`, Internal concentration resulting in 50% effect (ug L-1)
@@ -75,13 +81,21 @@ setClass("LemnaSetacScenario", contains="LemnaSetac")
 #'   - `k_met`, Metabolisation rate (d-1)
 #'
 #' @section Forcings:
+#' External concentrations, alias *exposure*,  have to be provided as *ug L-1*.
+#'
 #' Besides exposure, the model requires four environmental properties as
-#' time-series input:
-#' - `tmp`, temperature (°C)
+#' input:
+#' - `tmp`, ambient temperature (°C)
 #' - `irr`, irradiance (kJ m-2 d-1)
 #' - `P`, Phosphorus concentration (mg P L-1)
 #' - `N`, Nitrogen concentration (mg N L-1)
 #'
+#' The following constant default values are used for these properties:
+#' - `tmp` = 12 °C
+#' - `irr` = 15,000 kJ m-2 d-1
+#' - `P` = 0.3 mg L-1
+#' - `N` = 0.6 mg L-1
+#'
 #' Forcings time-series are represented by `data.frame` objects consisting of two
 #' columns. The first for time and the second for the environmental factor in question.
 #'
@@ -96,10 +110,10 @@ setClass("LemnaSetacScenario", contains="LemnaSetac")
 #' with biomass transfers.
 #'
 #' @section Simulation output:
-#' For reasons of convenience, the return value contains by default two additional
-#' variables derived from simulation results: the internal concentration `C_int`
-#' as well as the number of fronds `FrondNo`. These can be disabled by setting
-#' the argument `nout = 0`.
+#' For reasons of convenience, the return value of [simulate()] contains by
+#' default two additional variables derived from simulation results:
+#' the internal concentration `C_int` as well as the number of fronds `FrondNo`.
+#' These can be disabled by setting the argument `nout = 0`.
 #'
 #' The available output levels are as follows:
 #' - `nout >= 1`: `C_int`, internal concentration (mass per volume)
@@ -143,12 +157,20 @@ setClass("LemnaSetacScenario", contains="LemnaSetac")
 #'
 #' @inheritSection Transferable Biomass transfer
 #'
+#' @section Model history and changes:
+#' - cvasi v1.0.0
+#'   - Model and parameters as described by Klein et al. (2022) report version 1.1
+#' - cvasi v1.5.0
+#'   - Default value of parameter `beta` modified due to typo in previous report
+#'     versions, now conforms with Klein et al. (2025), report version 1.2
+#'   - New value: `beta=0.25`, old value: `beta=0.025`
+#'
 #' @references
 #' Klein J., Cedergreen N., Heine S., Reichenberger S., Rendal C.,
 #' Schmitt W., Hommen U., 2021: *Refined description of the Lemna TKTD growth model
 #' based on Schmitt et al. (2013) - equation system and default parameters*.
 #' Report of the working group *Lemna* of the SETAC Europe Interest Group Effect
-#' Modeling. Version 1, uploaded on 22. Sept. 2021.
+#' Modeling. Version 1.3, uploaded in July 2025.
 #' https://www.setac.org/group/effect-modeling.html
 #'
 #'
@@ -179,7 +201,7 @@ Lemna_SETAC <- function() {
         Q10 = 2,        # temperature coefficient (-)
         T_ref = 25,     # ref temperature for response=1 (°C)
         alpha = 5e-5,   # slope of irradiance response of photosynthesis (m2 d kJ-1)
-        beta = 0.025,   # intercept of irradiance response of photosynthesis (-)
+        beta = 0.25,    # intercept of irradiance response of photosynthesis (-)
         N_50 = 0.034,   # half-saturation constant of Nitrogen response (mg N L-1)
         P_50 = 0.0043,  # half-saturation constant of Phosphorus response (mg P L-1)
         BM_L = 177,     # carrying capacity (g dw m-2)
@@ -216,7 +238,7 @@ Lemna_SETAC <- function() {
       init=c(BM=0.0012, M_int=0),
       transfer.comp.biomass="BM",
       transfer.comp.scaled="M_int"
-  )
+  ) %>% set_noexposure()
 }
 
 
@@ -269,12 +291,12 @@ solver_lemna_setac <- function(scenario, nout=2, method="lsoda", hmax=0.1, ...)
                 "fP_photo", "fN_photo", "fBM_photo", "fCint_photo", "C_int_unb",
                 "C_ext", "Tmp", "Irr", "Phs", "Ntr", "dBM", "dM_int")
   # run deSolve
-  as.data.frame(ode(y=scenario@init, times=scenario@times, parms=params, dllname="cvasi",
-                    initfunc="lemna_setac_init", func="lemna_setac_func", initforc="lemna_setac_forc",
-                    forcings=forcings, nout=nout, outnames=outnames, method=method, hmax=hmax, ...))
+  ode(y=scenario@init, times=scenario@times, parms=params, dllname="cvasi",
+        initfunc="lemna_setac_init", func="lemna_setac_func", initforc="lemna_setac_forc",
+        forcings=forcings, nout=nout, outnames=outnames, method=method, hmax=hmax, ...)
 }
 #' @include solver.R
-#' @describeIn solver Numerically integrates Lemna_SETAC models
+#' @describeIn solver Numerically integrates Lemna_SETAC scenarios
 setMethod("solver", "LemnaSetac", function(scenario, ...) solver_lemna_setac(scenario, ...) )
 
 
@@ -290,6 +312,9 @@ fx_lemna <- function(scenario, ...) {
     stop("endpoint r is incompatible with biomass transfers")
 
   out <- simulate(scenario, ...)
+  if(!num_success(out)) {
+    return(NA)
+  }
 
   efx <- c("BM"=tail(out$BM, 1))
   if(efx_r) # we skip the log() operation if we can
