@@ -112,11 +112,11 @@ plot_sd <- function(model_base,
 
   if (!is.null(obs_full)) {
     obs_plot <- obs_full
-    obs_max <- max(obs_max, obs_plot[,2]) #2 = data, observations
+    obs_max <- max(obs_max, obs_plot[,2], na.rm=TRUE) #2 = data, observations
     obs_plot$trial <- as.factor(obs_plot$trial) #trial
   } else if (!is.null(obs_mean)) {
     obs_plot <- obs_mean
-    obs_max <- max(obs_max, obs_plot[,2])
+    obs_max <- max(obs_max, obs_plot[,2], na.rm=TRUE)
     obs_plot$trial <- as.factor(obs_plot$trial) #trial
   } else {
     obs_plot <- NULL
@@ -469,7 +469,7 @@ plot_ppc_combi <- function(table, xy_lim = NULL) {
 #'
 #' @param EPx_ts the result of `epx_mtw`, ie. a tibble with window.start,
 #' window.end, endpoint, level and EPx
-#' @param exposure_ts an exposure time series with columns for time 't' and
+#' @param exposure_ts an exposure time series with columns for time 'time' and
 #' concentration 'conc'
 #' @param draw Should the whole plot be drawn? If FALSE the exposure plot and
 #' the EPx plot are returned as a list for later modification
@@ -480,7 +480,7 @@ plot_ppc_combi <- function(table, xy_lim = NULL) {
 #'
 #' @return a grid of ggplots
 #' @export
-#' @global EPx len
+#' @autoglobal
 #'
 #' @examples
 #' ti <- 0:21
