@@ -128,7 +128,7 @@
 #' minnow_long %>% simulate(hmax=0.001, maxsteps=10^5)
 setGeneric("simulate", function(x, ...) standardGeneric("simulate"), signature="x")
 
-#' default for all models using sliding exposure windows
+#' Default for all models using moving exposure windows
 #' @rdname simulate
 #' @include class-EffectScenario.R
 setMethod("simulate", "EffectScenario", function(x, ...) simulate_scenario(x, ...))
@@ -148,12 +148,80 @@ setMethod("simulate", "SimulationBatch", function(x, ...) simulate_batch2(batch=
 
 # Wrapper for solver function to enforce setting of a S3 class for all simulation
 # results.
-simulate_scenario <- function(x, times, ...) {
-  if(!missing(times))
+#' @importFrom utils capture.output
+simulate_scenario <- function(x, times, .suppress=FALSE, ...) {
+  # set times vector, mostly for backwards compatibility
+  if(!missing(times)) {
     x <- set_times(x, times)
+  }
 
-  rs <- solver(scenario=x, ...)
-  class(rs) <- c("cvasi.simulate", class(rs))
+  # listof conditions raised by deSolve
+  has_error <- FALSE
+  has_warning <- FALSE
+  conds <- list()
+  out <- capture.output(
+           withCallingHandlers(
+             rs <- withRestarts(solver(scenario=x, ...), muffleStop=function(x) data.frame() ),
+             error=function(x) {
+               conds <<- append(conds, list(c("error", x$call, x$message)))
+               has_error <<- TRUE
+               invokeRestart("muffleStop")
+             },
+             warning=function(x) {
+               conds <<- append(conds, list(c("warning", x$call, x$message)))
+               has_warning <<- TRUE
+               tryInvokeRestart("muffleWarning")
+             })
+           )
+  # convert deSolve return type to an actual data.frame
+  if(is(rs, "deSolve")) {
+    rs <- ode2df(rs)
+  }
+  class(rs) <- c("cvasi_simulate", "data.frame")
+  was_aborted <- num_aborted(rs)
+  attr(rs, "cvasi_status") <- "success"
+
+  # suppress any conditions, this simplifies the calling code and also
+  # drastically reduces the time spent in this function
+  if(.suppress)
+  {
+    if(was_aborted)
+      attr(rs, "cvasi_status") <- "aborted"
+    else if(has_error)
+      attr(rs, "cvasi_status") <- "error"
+    else if(has_warning)
+      attr(rs, "cvasi_status") <- "warning"
+  }
+  else
+  {
+    # save any output messages from deSolve, i.e. error output
+    attr(rs, "desolve_output") <- out
+    attr(rs, "desolve_conds") <- conds
+    # set custom class for usability features such as plotting
+
+    attr(rs, "cvasi_status") <- "success"
+    if(was_aborted) {
+      attr(rs, "cvasi_status") <- "aborted"
+      warn1 <- which.min(sapply(conds, function(lst) lst[[1]] == "warning"))
+      if(length(warn1) == 0) {
+        conds <- list(list("", "", "An unknown error occurred."))
+        warn1 <- 1
+      }
+      warn(c("Simulation aborted.", conds[[warn1]][[3]]),
+           footer = "Please run `num_info()` on result to get help on numerical issues.")
+    }
+    else if(has_error) {
+      # find all error messages
+      err <- sapply(conds, function(lst) ifelse(lst[[1]] == "error", unlist(tail(lst, n=1)), NA_character_))
+      abort(c("Simulation failed", err[!is.na(err)]), class="cvasi_error")
+    }
+    else if(has_warning) {
+      attr(rs, "cvasi_status") <- "warning"
+      warn1 <- which.min(sapply(conds, function(lst) lst[[1]] == "warning"))
+      warn(c("Issues during simulation.", conds[[warn1]][[3]]),
+           footer = "Please run `num_info()` on result to get help on numerical issues.")
+    }
+  }
   rs
 }
 
@@ -167,12 +235,12 @@ simulate_scenario <- function(x, times, ...) {
 #
 # @param scenario EffectScenario to simulate
 # @param times optional numerical vector overriding the scenario's output times
-# @param in_sequence optional logical, set to TRUE if simulated scenario is part
+# @param .in_sequence optional logical, set to TRUE if simulated scenario is part
 #   of a scenario sequence, then it will return a proposed initial state for
 #   simulate_seq to use
 # @param ... additional parameters passend on to [solver()]
 # @return data.frame
-simulate_transfer <- function(scenario, times, in_sequence=FALSE, ...) {
+simulate_transfer <- function(scenario, times, .in_sequence=FALSE, ...) {
   if(!has_transfer(scenario)) # shortcut if no transfers were defined
     return(simulate_scenario(scenario, times=times, ...))
   if(!missing(times))
@@ -220,14 +288,17 @@ simulate_transfer <- function(scenario, times, in_sequence=FALSE, ...) {
   df <- data.frame()
   t_start <- t_min
   # add last time point to also simulate the remainder of the scenario, if it exists
-  if(!ends_on_transfer)
+  if(!ends_on_transfer) {
     tr_points <- c(tr_points, t_max)
+    # repeat the last value, although it will never be used
+    tr_biomass <- c(tr_biomass, tail(tr_biomass, n=1))
+  }
   # simulate population until next transfer to new medium
   for(i in seq_along(tr_points))
   {
     tr <- tr_points[i]
     period <- times[times >= t_start & times <= tr]
-    out <- solver(set_times(scenario, period), ...)
+    out <- simulate_scenario(set_times(scenario, period), ...)
 
     # select rows/time points:
     # 1) that do NOT occur directly after a transfer, i.e. the start of a simulation
@@ -247,20 +318,26 @@ simulate_transfer <- function(scenario, times, in_sequence=FALSE, ...) {
     init[[scenario@transfer.comp.biomass]] <- tr_biomass[i]
     # scale other compartments relative to new biomass
     init[scenario@transfer.comp.scaled] <- init[scenario@transfer.comp.scaled] * BM.fac
+    # in case of e.g. numerical issues, NaN can occur in sim output and new initial states
+    # TODO check if error due to e.g. numerics, to provide a targeted messages
+    if(any(is.nan(init))) {
+      warning("Simulation result contains NaN values, please check parameters and numerical stability.", call.=FALSE)
+      break
+    }
     scenario <- set_init(scenario, init)
     t_start <- tr
   }
 
   # add metadata at which system state the subsequent simulation should start
   # if they were simulated in a sequence, cf. simulate_seq()
-  if(in_sequence) {
+  if(.in_sequence) {
     if(ends_on_transfer)
       attr(df, "next_init") <- init
     else
       attr(df, "next_init") <- tail(df, 1)[names(scenario@init)]
   }
   rownames(df) <- NULL
-  class(df) <- c("cvasi.simulate", class(df))
+  class(df) <- c("cvasi_simulate", "data.frame")
   df
 }
 
@@ -272,51 +349,70 @@ simulate_transfer <- function(scenario, times, in_sequence=FALSE, ...) {
 #
 # @return data.frame
 simulate_seq <- function(seq, times, ...) {
-  if(length(seq@scenarios)==0)
-    stop("no scenarios in sequence")
-  if(!missing(times))
-    stop("overriding output times not supported for scenario sequences")
+  if(length(seq) == 0)
+    stop("No scenarios in sequence")
+  if(!missing(times)) {
+    seq <- set_times(seq, times)
+  }
+  # for backwards compatibility, include all breaks in output times
+  # since v1.5.0
+  if(methods::.hasSlot(seq, "inc_start")) {
+    inc_start <- seq@inc_start
+    inc_end <- seq@inc_end
+  } else {
+    inc_start <- c(TRUE, rep(FALSE, length(seq) - 1))
+    inc_end <- rep(TRUE, length(seq))
+  }
 
   df <- data.frame() # result table
-  init <- seq@scenarios[[1]]@init
-  t.start <- seq@scenarios[[1]]@times[1]
+  init <- seq[[1]]@init
+  t_start <- NA_real_
   nms <- names(init)
 
-  for(i in seq(length(seq@scenarios))) {
-    # get current scenario and re-set init state
-    sc <- set_init(seq@scenarios[[i]], init)
-    # if explicit time points were provided, try to accommodate them with the
-    # scenario's settings
-    if(!missing(times)) {
-      t.end <- sc@times[length(sc@times)]
-      sc.times <- times[times>=t.start & times<=t.end]
-      if(t.start < sc.times[1])
-        sc.times <- c(t.start, sc.times)
-      sc <- set_times(sc, sc.times)
+  for(i in seq_along(seq))
+  {
+    # skip scenario if no output times defined
+    if(length(seq[[i]]@times) == 0) {
+      next
     }
+    # get the first output time from the sequence
+    if(is.na(t_start)) {
+      t_start <- seq[[i]]@times[1]
+    }
+
+    # get current scenario and re-set init state
+    sc <- set_init(seq[[i]], init)
     # check if current scenario starts where the previous ended
-    if(sc@times[1] != t.start)
-      stop("scenarios are not on a continuous time scale")
-    t.start <- tail(sc@times,1)
+    if(sc@times[1] != t_start)
+      stop("Scenarios are not on a continuous time scale")
     # we simulate each scenario as is
-    out <- simulate(sc, times=times, in_sequence=TRUE, ...)
-    # append results to output data.frame
-    if(i==1)
-      df <- rbind(df, out)
-    else
-      df <- rbind(df, out[-1,])
+    out <- simulate(sc, .in_sequence=TRUE, ...)
 
     # which system state to use as initial state for next scenario
     # in sequence?
     # a) if simulate() returned suitable metadata then use that
     # b) otherwise, use last system state of last scenario
     init <- attr(out, "next_init", exact=TRUE)
-    if(is.null(init))
-      init <- tail(out,1)[nms]
+    if(is.null(init)) {
+      init <- tail(out, 1)[nms]
+    }
+
+    # filter certain output times from result if they are not supposed to be reported
+    if(!inc_start[[i]]) {
+      out <- out[sc@times != t_start, ]
+    }
+    if(!inc_end[[i]]) {
+      t_end <- sc@times[length(sc@times)]
+      out <- out[out[, 1] != t_end, ]
+    }
+
+    t_start <- tail(sc@times, 1)
+    # reported result
+    df <- rbind(df, out)
   }
   attr(df, "next_init") <- NULL
   rownames(df) <- NULL
-  class(df) <- c("cvasi.simulate", class(df))
+  class(df) <- c("cvasi_simulate", "data.frame")
   df
 }
 
@@ -390,6 +486,7 @@ simulate_batch2 <- function(batch, times, ...) {
 #' @param model_base effect scenario object with mean parameters
 #' @param treatments treatments exposure levels as data frame (time, conc, trial)
 #' @param param_sample *deprecated* parameter, no longer in use
+#' @param ... additional parameters passed through to [simulate()]
 #' @return a `data.frame` with simulation results
 #' @export
 #' @examples
@@ -399,8 +496,8 @@ simulate_batch2 <- function(batch, times, ...) {
 #'
 #' metsulfuron %>%
 #'   simulate_batch(treatments)
-simulate_batch <- function(model_base, treatments, param_sample=deprecated()) {
-  #lifecycle::deprecate_soft("1.4.0", "simulate_batch()", details="Please use `simulate(batch())` instead")
+simulate_batch <- function(model_base, treatments, param_sample=deprecated(), ...) {
+  lifecycle::deprecate_soft("1.5.0", "simulate_batch()", details="Please use `simulate(batch())` instead")
   if(is_present(param_sample)) {
     if(!is.null(param_sample)) {
       lifecycle::deprecate_stop("1.3.0", "simulate_batch(param_sample)")
@@ -428,8 +525,7 @@ simulate_batch <- function(model_base, treatments, param_sample=deprecated()) {
   # simulate
   df <- data.frame()
   df_one_run <- data.frame()
-  simulated_results <- purrr::map(list_of_effect_sets, ~ .x %>%
-                                    simulate())
+  simulated_results <- purrr::map(list_of_effect_sets, ~ .x %>% simulate(), ...)
   # add trial name
   simulated_results <- purrr::map2(
     simulated_results,
@@ -437,6 +533,6 @@ simulate_batch <- function(model_base, treatments, param_sample=deprecated()) {
     ~ dplyr::mutate(.x, trial = .y)
   )
   df <- dplyr::bind_rows(simulated_results)
-  class(df) <- c("cvasi.simulate", class(df))
+  class(df) <- c("cvasi_simulate", "data.frame")
   return(df)
 }
