@@ -19,6 +19,119 @@ window_IT <- function(df,window=-1,interval=-1) {
   c(tib[[1,"dat.start"]],tib[[1,"dat.end"]])
 }
 
+test_that("return types", {
+  sc <- minnow_it
+
+  expect_true(is.numeric(effect(sc, max_only=TRUE, ep_only=TRUE)))
+  expect_true(is.data.frame(effect(sc, max_only=TRUE, ep_only=FALSE)))
+  expect_true(is.data.frame(effect(sc, max_only=FALSE, ep_only=TRUE)))
+  expect_true(is.data.frame(effect(sc, max_only=FALSE, ep_only=FALSE)))
+  expect_true(is.numeric(effect(sc)$L))
+
+  sc <- metsulfuron
+  expect_true(is.numeric(effect(sc, max_only=TRUE, ep_only=TRUE)))
+  expect_true(is.data.frame(effect(sc, max_only=TRUE, ep_only=FALSE)))
+  expect_true(is.data.frame(effect(sc, max_only=FALSE, ep_only=TRUE)))
+  expect_true(is.data.frame(effect(sc, max_only=FALSE, ep_only=FALSE)))
+  expect_true(is.numeric(effect(sc)$BM))
+  expect_true(is.numeric(effect(sc)$r))
+})
+
+test_that("return names", {
+  # A GUTS-RED-IT model has only one effect endpoint
+  sc <- minnow_it
+
+  fxn <- function(...) {
+    names(effect(sc, ...))
+  }
+
+  expect_equal(fxn(ep_only=FALSE, max_only=FALSE), c("scenario","L","dat.start","dat.end"))
+  expect_equal(fxn(ep_only=FALSE, max_only=TRUE),  c("scenario","L","L.dat.start","L.dat.end"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=FALSE), c("L"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=TRUE),  c("L"))
+
+  expect_equal(fxn(ep_only=FALSE, max_only=FALSE, marginal_effect=1e-3), c("scenario","L","dat.start","dat.end"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=TRUE,  marginal_effect=1e-3), c("L"))
+
+  expect_equal(fxn(ep_only=FALSE, max_only=FALSE, factor=2), c("scenario","L","dat.start","dat.end"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=TRUE,  factor=2), c("L"))
+
+  # The Lemna model has two effect endpoints
+  sc <- metsulfuron
+
+  expect_equal(fxn(ep_only=FALSE, max_only=FALSE), c("scenario","BM","r","dat.start","dat.end"))
+  expect_equal(fxn(ep_only=FALSE, max_only=TRUE),  c("scenario","BM","BM.dat.start","BM.dat.end","r","r.dat.start","r.dat.end"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=FALSE), c("BM","r"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=TRUE),  c("BM","r"))
+
+  expect_equal(fxn(ep_only=FALSE, max_only=FALSE, marginal_effect=1e-3), c("scenario","BM","r","dat.start","dat.end"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=TRUE,  marginal_effect=1e-3), c("BM","r"))
+
+  expect_equal(fxn(ep_only=FALSE, max_only=FALSE, factor=2), c("scenario","BM","r","dat.start","dat.end"))
+  expect_equal(fxn(ep_only=TRUE,  max_only=TRUE,  factor=2), c("BM","r"))
+})
+
+test_that("factor argument", {
+  sc <- minnow_it
+  sc0 <- sc %>% set_noexposure()
+  sc10 <- sc %>% set_exposure(data.frame(t=0, c=10), reset_times=FALSE)
+  sc20 <- sc %>% set_exposure(data.frame(t=0, c=20), reset_times=FALSE)
+
+  expect_equal(effect(sc, factor=0, ep_only=TRUE), effect(sc0, ep_only=TRUE))
+  expect_equal(effect(sc, factor=10, ep_only=TRUE), effect(sc10, ep_only=TRUE))
+  expect_equal(effect(sc, factor=20, ep_only=TRUE), effect(sc20, ep_only=TRUE))
+})
+
+test_that("marginal effects", {
+  source(test_path("dummy.R"), local = TRUE)
+
+  # nothing changes if effect larger than threshold
+  sc <- new("DummyScenario", endpoints=c("foo","bar"), fx=c("foo"=1))
+  expect_equal(effect(sc, ep_only=TRUE, marginal_effect=1e-4), c(foo=1, bar=NA_real_))
+
+  # round effect value to zero
+  sc <- new("DummyScenario", endpoints=c("foo","bar"), fx=c("foo"=1e-5))
+  expect_equal(effect(sc, ep_only=TRUE, marginal_effect=1e-4), c(foo=0, bar=NA_real_))
+  sc <- new("DummyScenario", endpoints=c("foo","bar"), fx=c("foo"=-1e-5))
+  expect_equal(effect(sc, ep_only=TRUE, marginal_effect=1e-4), c(foo=0, bar=NA_real_))
+
+  sc <- new("DummyScenario", endpoints="foo", fx=c("foo"=-1e-5))
+  expect_equal(effect(sc, ep_only=TRUE, marginal_effect=1e-4), c(foo=0))
+
+  # test one 'real' scenario
+  sc <- minnow_it
+  expect_equal(effect(sc, ep_only=TRUE, marginal_effect=1e-4), c(L=0))
+
+  # questionable inputs
+  expect_warning(effect(sc, marginal_effect=0.1), "larger than 1%")
+})
+
+test_that("generic effect calculation", {
+  # no effect
+  expect_equal(calc_effect(1, 1), 0)
+  expect_equal(calc_effect(0, 0), 0)
+  expect_equal(calc_effect(-1, -1), 0)
+  # positive finite effects, i.e. treatment is lower than control
+  expect_equal(calc_effect(0.9, 1), 0.1)
+  expect_equal(calc_effect(0,  1), 1)
+  expect_equal(calc_effect(-1, 1), 2)
+
+  expect_equal(calc_effect(-1.1, -1), 0.1)
+  expect_equal(calc_effect(-2,   -1), 1)
+  expect_equal(calc_effect(-3,   -1), 2)
+  # negative finite effects, i.e. treatment is higher than control
+  expect_equal(calc_effect(1.1, 1), -0.1)
+  expect_equal(calc_effect(2,   1), -1)
+  expect_equal(calc_effect(3,   1), -2)
+
+  expect_equal(calc_effect(-0.9, -1), -0.1)
+  expect_equal(calc_effect( 0,   -1), -1)
+  expect_equal(calc_effect( 1,   -1), -2)
+  # infinite effects, i.e. control is zero
+  expect_equal(calc_effect(0.1,   0), -Inf)
+  expect_equal(calc_effect(-0.1,  0), Inf)
+})
+
 test_that("GUTS-RED pulsed exposure", {
   # allowed numerical tolerance between model runs
   tol <- 0.001
@@ -124,19 +237,19 @@ test_that("Lemna effects", {
 
   # check that r does not exceed 1.0 in case of negative growth rates
   metsulfuron %>%
-    set_exposure(data.frame(t=0:14,c=1000)) -> sc
+    set_exposure(data.frame(t=0:14, c=1000)) -> sc
   sc %>% simulate() -> out
   # check that we indeed have negative growth
-  expect_true(out$BM[1]>tail(out$BM,1))
-  # r effect endpoint should then max out at 1.0
+  expect_true(out$BM[1] > tail(out$BM,1))
+  # r effect endpoint should be greater than 1.0
   sc %>% effect() -> efx
-  expect_equal(efx %>% dplyr::pull(r), 1)
+  expect_gt(efx$r[1], 1)
   # all endpoints contained in output?
   expect_true(all(c("BM","r") %in% names(efx)))
 })
 
-test_that("Myriophyllum", {
-  Myrio() %>%
+test_that("Magma", {
+  Magma() %>%
     set_init(c(BM=1)) %>%
     set_exposure(metsulfuron@exposure@series) %>%
     set_param(c(EC50_int=0.5,b=0.2,P=0.01,r_DW_TSL=1)) %>%
@@ -166,28 +279,6 @@ test_that("general arguments", {
   expect_equal(effect(lemna, factor=factor), effect(lemna2), tolerance=1e-5)
 })
 
-test_that("marginal effects", {
-  # this DEB scenario creates a negative effect in one exposure window due
-  # to instable numerics
-  americamysis %>%
-    set_window(7) %>%
-    set_exposure(data.frame(t=c(0,3,4,7,8), c=c(0,0,3,3,0))) %>%
-    set_times(0:14) -> deb
-
-  # make sure a negative effect level exists
-  efx1 <- dplyr::pull(effect(deb, max_only=FALSE), "L")
-  expect_lt(min(efx1), 0)
-  expect_gt(min(efx1), -0.01)
-  # test marginal effect threshold
-  efx2 <- dplyr::pull(effect(deb, max_only=FALSE, marginal_effect=1e-5), "L")
-  expect_gte(min(efx2), 0)
-  expect_lte(max(abs(efx1 - efx2)), 1e-5)
-
-  # questionable inputs
-  expect_warning(effect(minnow_it, marginal_effect=0.1))
-
-})
-
 test_that("invalid arguments", {
   ## invalid inputs
   # multiple scenarios
