@@ -15,19 +15,13 @@ test_that("get_model", {
   expect_error(get_model(assessment()))
 })
 
-test_that("get_tag", {
-  s1 <- new("EffectScenario", tag="foo")
-  s2 <- new("EffectScenario", tag="bar")
-  p3 <- new("ParameterSet", tag="baz")
+test_that("get_window", {
+  sc <- new("EffectScenario") %>% set_window(length=-1, interval=-1)
+  expect_equal(get_window(sc), list(length=-1, interval=-1))
 
-  # empty argument
-  expect_equal(get_tag(list()), list())
-  # single argument
-  expect_equal(get_tag(s1), s1@tag)
-  expect_equal(get_tag(p3), p3@tag)
-  # vectorized arguments
-  expect_equal(get_tag(c(s1,s2)), c(s1@tag, s2@tag))
-  expect_equal(get_tag(c(s1,p3)), c(s1@tag, p3@tag))
-  # invalid arguments
-  expect_error(get_tag(assessment()))
+  sc <- new("EffectScenario") %>% set_window(length=2, interval=3)
+  l <- list(length=2, interval=3)
+  expect_equal(get_window(sc), l)
+
+  expect_equal(get_window(list(sc, sc)), list(l, l))
 })
