@@ -4,7 +4,6 @@ test_that("has_forcings", {
   expect_equal(has_forcings(c(scenario,scenario)), c(TRUE,TRUE))
   # no forcings
   expect_false(has_forcings(GUTS_RED_IT()))
-  expect_false(has_forcings(Lemna_Schmitt()))
   # invalid arguments
   expect_error(has_forcings(NA))
   expect_error(has_forcings(NULL))
@@ -100,10 +99,10 @@ test_that("has_regular_transfer", {
   expect_false(has_regular_transfer(Lemna_Schmitt() %>% set_transfer(interval=-1)))
   expect_false(has_regular_transfer(Lemna_Schmitt() %>% set_transfer(times=1)))
   # invalid arguments
-  expect_error(has_regular_transfer(NA))
-  expect_error(has_regular_transfer(NULL))
-  expect_error(has_regular_transfer(1))
-  expect_error(any(has_regular_transfer(1:5)))
+  expect_false(has_regular_transfer(NA))
+  expect_false(has_regular_transfer(NULL))
+  expect_false(has_regular_transfer(1))
+  expect_false(any(has_regular_transfer(1:5)))
 })
 
 test_that("has_irregular_transfer", {
@@ -113,8 +112,19 @@ test_that("has_irregular_transfer", {
   expect_false(has_irregular_transfer(Lemna_Schmitt() %>% set_transfer(interval=-1)))
   expect_false(has_irregular_transfer(Lemna_Schmitt() %>% set_transfer(interval=1)))
   # invalid arguments
-  expect_error(has_irregular_transfer(NA))
-  expect_error(has_irregular_transfer(NULL))
-  expect_error(has_irregular_transfer(1))
-  expect_error(any(has_irregular_transfer(1:5)))
+  expect_false(has_irregular_transfer(NA))
+  expect_false(has_irregular_transfer(NULL))
+  expect_false(has_irregular_transfer(1))
+  expect_false(any(has_irregular_transfer(1:5)))
+})
+
+test_that("has_units", {
+  bar <- 1
+  foo <- units::set_units(1, sec)
+
+  expect_true(has_units(foo))
+  expect_true(all(has_units(list(foo, foo))))
+  expect_false(has_units(bar))
+  expect_false(any(has_units(list(bar, bar))))
+  expect_true(any(has_units(list(foo, bar))))
 })
