@@ -1,30 +1,33 @@
-test_that("get/set on scenario", {
+test_that("scenario", {
   # single boundary
   bds <- list(kd=c(2, 3))
-  sc <- GUTS_RED_IT() %>% set_bounds(bds)
+  sc <- new("EffectScenario") %>% set_bounds(bds)
   expect_equal(get_bounds(sc), bds)
 
+  # update boundary
   bds <- list(kd=c(23, 24))
-  sc <- GUTS_RED_IT() %>% set_bounds(bds)
+  sc <- sc %>% set_bounds(bds)
   expect_equal(get_bounds(sc), bds)
 
   # multiple boundaries
   bds <- list(kd=c(0, 1), hb=c(2, 3))
-  sc <- GUTS_RED_IT() %>% set_bounds(bds)
+  sc <- new("EffectScenario") %>% set_bounds(bds)
   expect_equal(get_bounds(sc), bds)
-
-  # update boundaries
-  GUTS_RED_IT() %>%
-    set_bounds(list(hb=c(0, 1))) %>%
-    set_bounds(list(hb=c(3, 4))) -> sc
-    expect_equal(get_bounds(sc), list(hb=c(3,4)))
 })
 
-test_that("on caliset", {
-  bds <- list(kd=c(23,24))
-  cs <- caliset(GUTS_RED_IT(), data.frame(t=0, a=1))
+test_that("sequence", {
+  bds <- list(kd=c(23, 24))
+  sc <- new("EffectScenario") %>% set_times(0:5)
+  suppressWarnings(sq <- sequence(list(sc)))
+  sq <- set_bounds(sq, bds)
+  expect_equal(get_bounds(sq), bds)
+})
+
+test_that("caliset", {
+  bds <- list(kd=c(23, 24))
+  cs <- caliset(new("EffectScenario"), data.frame(t=0, a=1))
   cs <- set_bounds(cs, bds)
-  expect_equal(get_bounds(cs@scenario), bds)
+  expect_equal(get_bounds(cs), bds)
 })
 
 test_that("list of scenarios", {
@@ -57,16 +60,3 @@ test_that("invalid arguments", {
   # wrong order
   expect_warning(expect_error(set_bounds(sc, list(kd=c(3, 2)))))
 })
-
-test_that("get_bounds", {
-  sc <- new("EffectScenario")
-  sc@param.req <- c("foo", "bar")
-
-  bnds <- list(foo=c(1, 3))
-  sc <- set_bounds(sc, bnds)
-  expect_equal(get_bounds(sc), bnds)
-
-  # invalid arguments
-  expect_error(get_bounds("foo"))
-  expect_error(get_bounds(c(sc, sc)))
-})
