@@ -18,14 +18,10 @@ test_that("basic arguments", {
   expect_equal(rs@exposure@series, es@series)
   expect_equal(rs@times, 1:3)
 
-  # single scenario, data.frame containing column with exposure series
-  df <- tibble::tibble(foo="bar", baz=1, series=c(es))
-  rs <- set_exposure(sc, df, reset_times=FALSE)
-  expect_type(rs, "list")
-  expect_equal(length(rs), 1)
-  expect_true(all(is_scenario(rs)))
-  expect_equal(rs[[1]]@exposure@series, es@series)
-  expect_equal(rs[[1]]@times, 1:3)
+  # drop additional columns
+  expect_warning(rs <- set_exposure(sc, data.frame(a=0, b=1, c=2), reset_times=FALSE))
+  expect_equal(length(rs@exposure@series), 2)
+  expect_equal(names(rs@exposure@series), c("a", "b"))
 })
 
 test_that("vectorized arguments", {
@@ -75,16 +71,37 @@ test_that("vectorized arguments", {
   expect_equal(rs[[4]]@name,  "b")
 })
 
+test_that("scenario sequence", {
+  sc <- minnow_it %>% set_noexposure()
+  suppressMessages(seq <- sequence(list(sc, sc), breaks=2))
+
+  df <- data.frame(foo=1, bar=2)
+  seq <- seq %>% set_exposure(df, reset_times=FALSE)
+  expect_equal(seq@scenarios[[1]]@exposure@series, df)
+  expect_equal(seq@scenarios[[2]]@exposure@series, df)
+})
+
 test_that("set_noexposure", {
   sc <- new("EffectScenario", name="a") %>%
     set_exposure(data.frame(time=1, conc=2), FALSE) %>%
     set_times(1:3)
 
+  rs <- sc %>% set_exposure(no_exposure())
+  # times unmodified
+  expect_equal(rs@times, sc@times)
+  # exposure set to zero
+  expect_true(all(rs@exposure@series[,2] == 0))
+
   rs <- set_noexposure(sc)
   # times unmodified
   expect_equal(rs@times, sc@times)
   # exposure set to zero
   expect_true(all(rs@exposure@series[,2] == 0))
+
+  # multiple scenarios at once
+  rs <- list(sc, sc) %>% set_noexposure()
+  expect_true(all(rs[[1]]@exposure@series[,2] == 0))
+  expect_true(all(rs[[2]]@exposure@series[,2] == 0))
 })
 
 test_that("non-standard argument types", {
@@ -99,17 +116,15 @@ test_that("non-standard argument types", {
   units(df$conc) <- units::as_units("ug/L")
   sc <- set_exposure(new("EffectScenario", name="a"), df)
   # units must not be present afterwards
-  expect_false(any(has_units(sc@exposure@series[,1])))
-  expect_false(any(has_units(sc@exposure@series[,2])))
+  expect_false(any(has_units(sc@exposure@series[, 1])))
+  expect_false(any(has_units(sc@exposure@series[, 2])))
 })
 
 test_that("invalid arguments", {
   sc <- minnow_it
   es <- sc@exposure
   # exposure series too short
-  expect_warning(set_exposure(sc, data.frame(t=0, c=0)))
-  suppressWarnings(sc2 <- set_exposure(sc, data.frame(t=0, c=0)))
-  expect_equal(sc2@times, sc@times)
+  expect_error(set_exposure(sc, data.frame(t=0, c=0)))
 
   # nonsense
   expect_error(set_exposure(sc, 1))
@@ -120,3 +135,62 @@ test_that("invalid arguments", {
   expect_error(set_exposure(1, data.frame(time="1")))
   expect_error(set_exposure(1, data.frame(time=1, conc="a")))
 })
+
+test_that("check_exposure", {
+  # no series
+  expect_error(check_exposure())
+  expect_error(check_exposure(foo))
+  expect_error(check_exposure(1))
+  expect_error(check_exposure(NULL))
+  # too few columns
+  expect_error(check_exposure(data.frame()), "two columns")
+  expect_error(check_exposure(data.frame(a=0)), "two columns")
+  # empty series
+  expect_error(check_exposure(data.frame(t=numeric(0), e=numeric(0))), "least one row")
+  # non-numeric columns
+  expect_error(check_exposure(data.frame(t="foo", e=0)), "numeric values")
+  expect_error(check_exposure(data.frame(t=0, e="bar")), "numeric values")
+  expect_error(check_exposure(data.frame(t="foo", e="bar")), "numeric values")
+  # missing/invalid values
+  expect_error(check_exposure(data.frame(t=NA_real_, e=0)), "invalid values")
+  expect_error(check_exposure(data.frame(t=Inf, e=0)), "invalid values")
+  expect_error(check_exposure(data.frame(t=NaN, e=0)), "invalid values")
+  expect_error(check_exposure(data.frame(t=0, e=NA_real_)), "invalid values")
+  expect_error(check_exposure(data.frame(t=0, e=Inf)), "invalid values")
+  expect_error(check_exposure(data.frame(t=0, e=NaN)), "invalid values")
+  # time not sorted
+  expect_error(check_exposure(data.frame(t=c(0,1,0), e=0)), "ascending order")
+
+  # additional columns
+  expect_warning(check_exposure(data.frame(t=0, e=0, f=0)), "additional columns")
+
+  # valid argument
+  check_exposure(data.frame(t=0:3, e=0))
+  # valid arguments with units
+  foo <- units::set_units(1, sec)
+  check_exposure(data.frame(t=c(foo, foo), e=0))
+  check_exposure(data.frame(t=0, e=c(foo, foo)))
+})
+
+test_that("ExposureSeries constructor", {
+  # shortened test set
+  expect_error(ExposureSeries())
+  expect_error(ExposureSeries(1))
+  expect_error(ExposureSeries(data.frame()), "two columns")
+  expect_error(ExposureSeries(data.frame(t=numeric(0), e=numeric(0))), "least one row")
+  expect_error(ExposureSeries(data.frame(t="foo", e=0)), "numeric values")
+  expect_error(ExposureSeries(data.frame(t=NA_real_, e=0)), "invalid values")
+  expect_warning(es <- ExposureSeries(data.frame(t=0, e=0, f=0)), "additional columns")
+  # where additional columns dropped?
+  expect_equal(length(es@series), 2)
+
+  # valid arguments
+  df <- data.frame(t=0:3, e=0)
+  es <- ExposureSeries(df)
+  expect_equal(es@series, df)
+  # valid arguments with units
+  foo <- units::set_units(1, sec)
+  df <- data.frame(t=c(foo, foo), e=0)
+  es <- ExposureSeries(df)
+  expect_equal(es@series, df)
+})
